@@ -4,7 +4,6 @@ from copy import deepcopy
 from importlib import import_module
 from pathlib import Path
 import json
-import logging
 import os
 import shutil
 import sys
@@ -22,8 +21,7 @@ from hidebound.exporters.local_disk_exporter import LocalDiskExporter
 from hidebound.exporters.s3_exporter import S3Exporter
 import hidebound.core.database_tools as db_tools
 import hidebound.core.tools as tools
-
-LOGGER = logging.getLogger(__name__)
+from hidebound.core.logging import ProgressLogger
 # ------------------------------------------------------------------------------
 
 
@@ -138,6 +136,20 @@ class Database:
         Returns:
             Database: Database instance.
         '''
+        # validate hidebound dir
+        hb_root = Path(hidebound_dir)
+        if not hb_root.is_dir():
+            msg = f'{hb_root} is not a directory or does not exist.'
+            raise FileNotFoundError(msg)
+
+        if Path(hb_root).name != 'hidebound':
+            msg = f'{hb_root} directory is not named hidebound.'
+            raise NameError(msg)
+
+        # setup logger
+        logpath = Path(hb_root, 'logs', 'progress', 'hidebound-progress.log')
+        self._logger = ProgressLogger(__name__, logpath)
+
         # validate spec classes
         bad_specs = list(filter(
             lambda x: not issubclass(x, SpecificationBase), specifications
@@ -145,29 +157,23 @@ class Database:
         if len(bad_specs) > 0:
             msg = 'SpecificationBase may only contain subclasses of '
             msg += f'SpecificationBase. Found: {bad_specs}.'
+            self._logger.error(msg)
             raise TypeError(msg)
 
         # validate root dir
         root = Path(root_dir)
         if not root.is_dir():
             msg = f'{root} is not a directory or does not exist.'
+            self._logger.error(msg)
             raise FileNotFoundError(msg)
 
         # validate write mode
         modes = ['copy', 'move']
         if write_mode not in modes:
             msg = f'Invalid write mode: {write_mode} not in {modes}.'
+            self._logger.error(msg)
             raise ValueError(msg)
 
-        # validate hidebound dir
-        hb_root = Path(hidebound_dir)
-        if not hb_root.is_dir():
-            msg = f'{hb_root} is not a directory or does not exist.'
-            raise FileNotFoundError(msg)
-        if Path(hb_root).name != 'hidebound':
-            msg = f'{hb_root} directory is not named hidebound.'
-            raise NameError(msg)
-
         self._root = root
         self._hb_root = hb_root
         self._include_regex = include_regex
@@ -182,6 +188,8 @@ class Database:
         # needed for testing
         self.__exporter_lut = None
 
+        self._logger.info('Database initialized')
+
     def create(self):
         # type: () -> "Database"
         '''
@@ -192,8 +200,10 @@ class Database:
 
             * file content to hb_parent/hidebound/content - under same directory
                                                             structure
-            * file metadata as json to hb_parent/hidebound/metadata/file
             * asset metadata as json to hb_parent/hidebound/metadata/asset
+            * file metadata as json to hb_parent/hidebound/metadata/file
+            * asset log as json to hb_parent/hidebound/logs/asset
+            * file log as json to hb_parent/hidebound/logs/file
 
         Raises:
             RunTimeError: If data has not been initialized.
@@ -201,6 +211,7 @@ class Database:
         Returns:
             Database: self.
         '''
+        total = 7
         if self.data is None:
             msg = 'Data not initialized. Please call update.'
             raise RuntimeError(msg)
@@ -209,17 +220,23 @@ class Database:
             with open(filepath, 'w') as f:
                 json.dump(obj, f)
 
+        def write_log(log, filepath):
+            with open(filepath, 'w') as f:
+                f.write(log)
+
         temp = db_tools._get_data_for_write(
             self.data, self._root, self._hb_root
         )
+        self._logger.info('create: get data', step=1, total=total)
         if temp is None:
             return self
 
-        file_data, file_meta, asset_meta = temp
+        file_data, asset_meta, file_meta, asset_log, file_log = temp
 
         # make directories
         for item in temp:
             item.target.apply(lambda x: os.makedirs(Path(x).parent, exist_ok=True))
+        self._logger.info('create: make directories', step=2, total=total)
 
         # write file data
         if self._write_mode == 'move':
@@ -227,13 +244,25 @@ class Database:
             tools.delete_empty_directories(self._root)
         else:
             file_data.apply(lambda x: shutil.copy2(x.source, x.target), axis=1)
-
-        # write file metadata
-        file_meta.apply(lambda x: write_json(x.metadata, x.target), axis=1)
+        self._logger.info('create: write file data', step=3, total=total)
 
         # write asset metadata
         asset_meta.apply(lambda x: write_json(x.metadata, x.target), axis=1)
+        self._logger.info('create: write asset metadata', step=4, total=total)
 
+        # write file metadata
+        file_meta.apply(lambda x: write_json(x.metadata, x.target), axis=1)
+        self._logger.info('create: write file metadata', step=5, total=total)
+
+        # write asset log
+        asset_log.apply(lambda x: write_log(x.metadata, x.target), axis=1)
+        self._logger.info('create: write asset log', step=6, total=total)
+
+        # write file log
+        file_log.apply(lambda x: write_log(x.metadata, x.target), axis=1)
+        self._logger.info('create: write file log', step=7, total=total)
+
+        self._logger.info('create: complete', step=7, total=total)
         return self
 
     def read(self, group_by_asset=False):
@@ -252,6 +281,7 @@ class Database:
         Returns:
             DataFrame: Formatted data.
         '''
+        total = 5
         if self.data is None:
             msg = 'Data not initialized. Please call update.'
             raise RuntimeError(msg)
@@ -265,11 +295,13 @@ class Database:
             return item
 
         data = self.data.copy()
+        self._logger.info('read: copy data', step=1, total=total)
 
         col = 'file_traits'
         if group_by_asset:
             col = 'asset_traits'
             data = data.groupby('asset_path', as_index=False).first()
+            self._logger.info('read: group by asset', step=2, total=total)
 
         data[col] = data[col].apply(coordinate_to_dict)
         traits = DataFrame(data[col].tolist())
@@ -280,6 +312,7 @@ class Database:
 
             mask = traits[col].notnull()
             data.loc[mask, col] = traits.loc[mask, col]
+        self._logger.info('read: filter traits', step=3, total=total)
 
         # find columns by legal type
         cols = self.data.columns.tolist()
@@ -288,8 +321,9 @@ class Database:
                 .applymap(type) \
                 .apply(lambda x: x.unique().tolist())
             legal_cols = set([int, float, str, bool, None])
-            mask = cols.apply(lambda x: set(x).difference(legal_cols) == set())
-            cols = cols[mask].index.tolist()
+            cols = cols.apply(lambda x: set(x).difference(legal_cols) == set())
+            cols = cols[cols].index.tolist()
+            self._logger.info('read: filter legal types', step=4, total=total)
 
         # nicely order columns
         head_cols = [
@@ -315,7 +349,9 @@ class Database:
         tail_cols = sorted(list(set(cols).difference(head_cols)))
         cols = head_cols + tail_cols
         data = data[cols]
+        self._logger.info('read: order columns', step=5, total=total)
 
+        self._logger.info('read: complete', step=5, total=total)
         return data
 
     def update(self):
@@ -327,25 +363,53 @@ class Database:
         Returns:
             Database: self.
         '''
+        total = 12
+        self._logger.info('update', step=0, total=total)
+
+        exclude_re = '|'.join([self._exclude_regex, 'hidebound/logs/progress'])
         data = tools.directory_to_dataframe(
             self._root,
             include_regex=self._include_regex,
-            exclude_regex=self._exclude_regex
+            exclude_regex=exclude_re
         )
+        self._logger.info(f'update: parsed {self._root}', step=1, total=total)
+
         if len(data) > 0:
             db_tools._add_specification(data, self._specifications)
+            self._logger.info('update: add_specification', step=2, total=total)
+
             db_tools._validate_filepath(data)
+            self._logger.info('update: validate_filepath', step=3, total=total)
+
             db_tools._add_file_traits(data)
+            self._logger.info('update: add_file_traits', step=4, total=total)
+
             db_tools._add_relative_path(data, 'filepath', self._root)
+            self._logger.info('update: add_relative_path', step=5, total=total)
+
             db_tools._add_asset_name(data)
+            self._logger.info('update: add_asset_name', step=6, total=total)
+
             db_tools._add_asset_path(data)
+            self._logger.info('update: add_asset_path', step=7, total=total)
+
             db_tools._add_relative_path(data, 'asset_path', self._root)
+            self._logger.info('update: add_relative_path', step=8, total=total)
+
             db_tools._add_asset_type(data)
+            self._logger.info('update: add_asset_type', step=9, total=total)
+
             db_tools._add_asset_traits(data)
+            self._logger.info('update: add_asset_traits', step=10, total=total)
+
             db_tools._validate_assets(data)
+            self._logger.info('update: validate_assets', step=11, total=total)
 
         data = db_tools._cleanup(data)
         self.data = data
+
+        self._logger.info('update: cleanup', step=12, total=total)
+        self._logger.info('update: complete', step=12, total=total)
         return self
 
     def delete(self):
@@ -357,14 +421,18 @@ class Database:
         Returns:
             Database: self.
         '''
+        total = 2
         data_dir = Path(self._hb_root, 'content')
         if data_dir.exists():
             shutil.rmtree(data_dir)
+        self._logger.info('delete: data directory', step=1, total=total)
 
         meta_dir = Path(self._hb_root, 'metadata')
         if meta_dir.exists():
             shutil.rmtree(meta_dir)
+        self._logger.info('delete: metadata directory', step=2, total=total)
 
+        self._logger.info('delete: complete', step=2, total=total)
         return self
 
     def call_webhooks(self):
@@ -375,7 +443,8 @@ class Database:
         Yields:
             requests.Response: Webhook response.
         '''
-        for hook in self._webhooks:
+        total = len(self._webhooks)
+        for i, hook in enumerate(self._webhooks):
             url = hook['url']
             headers = hook.get('headers', None)
             method = hook['method']
@@ -391,13 +460,18 @@ class Database:
                 kwargs['params'] = hook['params']
 
             method = getattr(requests, method)
-            yield method(url, headers=headers, **kwargs)
+            response = method(url, headers=headers, **kwargs)
+            self._logger.info(f'call_webhooks: {url}', step=i, total=total)
+            yield response
 
     def export(self):
-        # type: () -> None
+        # type: () -> "Database"
         '''
         Exports all the files found in in hidebound root directory.
         Calls webhooks afterwards.
+
+        Returns:
+            Database: Self.
         '''
         # TODO: Find a nicer pattern for injecting exporters.
         lut = dict(
@@ -414,16 +488,19 @@ class Database:
         order = ['local_disk', 's3', 'girder']
         items = sorted(self._exporters.items(), key=lambda x: order.index(x[0]))
 
-        for key, config in items:
+        total = len(items)
+        for i, (key, config) in enumerate(items):
             exporter = lut[key].from_config(config)
             exporter.export(self._hb_root)
+            self._logger.info('export: export item', step=i, total=total)
 
             # assign instance to exporter_lut for testing
             if self.__exporter_lut is not None:
                 self.__exporter_lut[key] = exporter
 
-        for response in self.call_webhooks():
-            LOGGER.info(response.content)  # pragma: no cover
+        list(self.call_webhooks())
+        self._logger.info('export: complete')
+        return self
 
     def search(self, query, group_by_asset=False):
         # type: (str, bool) -> "DataFrame"
@@ -438,7 +515,10 @@ class Database:
         Returns:
             DataFrame: Formatted data.
         '''
-        return pandasql.sqldf(
+        result = pandasql.sqldf(
             query,
             {'data': self.read(group_by_asset=group_by_asset)}
         )
+        self._logger.info(f'search: {query}', step=1, total=1)
+        self._logger.info('search: complete', step=1, total=1)
+        return result
