@@ -535,3 +535,105 @@ def coordinates_begin_at(items, origin):
         return True
     msg = f'Coordinates do not begin at {origin}.'
     raise ValidationError(msg)
+
+
+@validate('''{} is not a valid bucket name. Bucket names must:
+    - be between 3 and 63 characters
+    - only consist of lowercase letters, numbers, periods and hyphens
+    - begin and end with a letter or number''')
+def is_bucket_name(item):
+    # type: (str) -> bool
+    '''
+    Validates a bucket name.
+
+    Args:
+        item (str): bucket name.
+
+    Raises:
+        ValidationError: If bucket name is invalid.
+
+    Returns:
+        bool: Validity of bucket name.
+    '''
+    if not 3 <= len(item) <= 63:
+        return False
+    if not item.islower():
+        return False
+    if re.search('^[a-z0-9][a-z0-9-.]*[a-z0-9]$', item) is None:
+        return False
+    return True
+
+
+@validate('{} is not a valid AWS region.')
+def is_aws_region(item):
+    # type: (str) -> bool
+    '''
+    Validates an AWS region name.
+
+    Args:
+        item (str): AWS region name.
+
+    Raises:
+        ValidationError: If region name is invalid.
+
+    Returns:
+        bool: Validity of region name.
+    '''
+    # list derived from boto.session.Session().get_available_regions('s3')
+    regions = [
+        'af-south-1',
+        'ap-east-1',
+        'ap-northeast-1',
+        'ap-northeast-2',
+        'ap-northeast-3',
+        'ap-south-1',
+        'ap-southeast-1',
+        'ap-southeast-2',
+        'ca-central-1',
+        'eu-central-1',
+        'eu-north-1',
+        'eu-south-1',
+        'eu-west-1',
+        'eu-west-2',
+        'eu-west-3',
+        'me-south-1',
+        'sa-east-1',
+        'us-east-1',
+        'us-east-2',
+        'us-west-1',
+        'us-west-2',
+    ]
+    return item in regions
+
+
+@validate('''{} is not a legal directory path.
+Legal directory paths must:
+    - Begin with /
+    - Not end with /
+    * Contain only the characters: /, a-z, A-Z, 0-9, _, -''')
+def is_legal_directory(item):
+    # type: (str) -> bool
+    '''
+    Validates that directory path is legal.
+    Legal directory paths must:
+
+        * Begin with /
+        * Not end with /
+        * Contain only the characters: /, a-z, A-Z, 0-9, _, -
+
+    Args:
+        item (str): Directory path.
+
+    Raises:
+        ValidationError: If directory path is invalid.
+
+    Returns:
+        bool: Validity of directory path.
+    '''
+    if not item.startswith('/'):
+        return False
+    if item.endswith('/'):
+        return False
+    if not re.search(r'^[/a-z0-9_\-]+$', item, re.I):
+        return False
+    return True
