@@ -19,6 +19,9 @@ class ExporterBase:
             * metadata
             * metadata/asset
             * metadata/file
+            * logs
+            * logs/asset
+            * logs/file
 
         Args:
             hidebound_dir (Path or str): Hidebound directory.
@@ -30,7 +33,10 @@ class ExporterBase:
         meta = Path(hidebound_dir, 'metadata')
         asset_dir = Path(meta, 'asset')
         file_dir = Path(meta, 'file')
-        for path in [data, meta, asset_dir, file_dir]:
+        logs = Path(hidebound_dir, 'logs')
+        asset_log = Path(logs, 'asset')
+        file_log = Path(logs, 'file')
+        for path in [data, meta, asset_dir, file_dir, logs, asset_log, file_log]:
             if not path.is_dir():
                 msg = f'{path.as_posix()} directory does not exist.'
                 raise FileNotFoundError(msg)
@@ -64,6 +70,19 @@ class ExporterBase:
                     file_meta = jsonc.JsonComment().load(f)
                 self._export_file(file_meta)
 
+        # export logs
+        for kind in ['asset', 'file']:
+            log_path = Path(hidebound_dir, 'logs', kind)
+            for filename in os.listdir(log_path):
+                filepath = Path(log_path, filename)
+                with open(filepath) as f:
+                    log = dict(filename=filepath.name, content=f.read())
+
+                if kind == 'asset':
+                    self._export_asset_log(log)
+                else:
+                    self._export_file_log(log)
+
     def _export_asset(self, metadata):
         # type: (Dict) -> None
         '''
@@ -91,3 +110,31 @@ class ExporterBase:
         '''
         msg = '_export_file method must be implemented in subclass.'
         raise NotImplementedError(msg)
+
+    def _export_asset_log(self, metadata):
+        # type: (Dict[str, str]) -> None
+        '''
+        Exports content from asset log in hidebound/logs/asset.
+
+        Args:
+            metadata (dict): Asset log.
+
+        Raises:
+            NotImplementedError: If method is not implemented in subclass.
+        '''
+        msg = '_export_asset_log method must be implemented in subclass.'
+        raise NotImplementedError(msg)
+
+    def _export_file_log(self, metadata):
+        # type: (Dict[str, str]) -> None
+        '''
+        Exports content from file log in hidebound/logs/file.
+
+        Args:
+            metadata (dict): File log.
+
+        Raises:
+            NotImplementedError: If method is not implemented in subclass.
+        '''
+        msg = '_export_file_log method must be implemented in subclass.'
+        raise NotImplementedError(msg)
